
// Written by Hans de Nivelle, Dec 2014.

#include "airfoil_mach0_fullrange.h"
#include <stdexcept>


coefficients
airfoil_mach0_fullrange::getcoefficients( double alpha ) const
{
   // We need alpha within -180.0 and +180.0 degrees.

   if( alpha < -180.0_deg || alpha > 180.0_deg )
      throw std::logic_error( "getcoefficients: alpha out of range" );

   // If alpha is within the domain of spec, we take spec.

   if( spec. alpha0 <= alpha && alpha <= spec. alpha1 )
   {
      return spec. getcoefficients( alpha );
   }

   // If alpha is sufficiently close to the domain of spec,
   // we take a linear interpolation between the default and
   // the border value of spec. `Sufficiently close' is defined by
   // transition. 

   const double transition = 15.0_deg;

   if( spec. alpha0 - transition <= alpha && alpha < spec. alpha0 ) 
   { 
      double rat = ( alpha - spec. alpha0 + transition ) / transition;
      coefficients c1 = def. getcoefficients( alpha );
      coefficients c2 = spec. getcoefficients( spec. alpha0 );
      c2. alpha = alpha; 
      return coefficients::interpolate( rat, c1, c2 );
   }

   if( spec. alpha1 < alpha && alpha < spec. alpha1 + transition )
   {
      double rat = ( alpha - spec. alpha1 ) / transition;
      coefficients c1 = spec. getcoefficients( spec. alpha1 );
      c1. alpha = alpha; 
      coefficients c2 = def. getcoefficients( alpha );
      return coefficients::interpolate( rat, c1, c2 );
   }

   // In the remaining case, we return the default:

   return def. getcoefficients( alpha ); 
}


