
#ifndef ATTITUDE_INCLUDED
#define ATTITUDE_INCLUDED 1

#include "linalg/quaternion.h"

// Construct quaternion that gives the orientation of the plane
// in body coordinates, taking body coordinates as starting points. 
//
// phi: Direction of plane (0 is north, pi/4 is east.) 
// theta: Pitch of plane. (0 is flat, pi/4 is straight up.)
// psi: Roll of plane. (0 is flat, pi/4 is vertical to the right.)
//
// In order to get the orientation relative to LENU, use
// northaligned( ) * getattitude( ). 

linalg::quaternion getattitude( double phi, double theta, double psi );

inline linalg::quaternion northaligned( ) { return { 0, 1,1,0 }; }
   // Orientation of plane, when it stands on the ground,
   // pointing north in the LENU frame.

#endif

