
#ifndef LINALG_NORMDEF_INCLUDED
#define LINALG_NORMDEF_INCLUDED  1

#include "quaternion.h"
// Because it is a specialization.

//  normdef<X> solves two problems:
//    How to define the norm |X| for arbitrary type X. 
//    How to get the object with |X| = 0.

namespace linalg
{

   // A normdef<X> object can be applied to create the norm of X.
   // The default behavior is to call x. norm( ), and to use
   // X{ } for the zero. 
 
   template< typename X >
   struct normdef
   {
      inline X zero( ) const { return X{ }; } 
      inline double operator( ) ( X x ) const { return x. norm( ); }
   };


   // For a double/float, norm(d) = d * d.

   template< >
   struct normdef< double >
   {
      inline double zero( ) const { return 0; } 
      inline double operator( ) ( double d ) const { return d*d; } 
   };

   template< >
   struct normdef< float >
   {
      inline float zero( ) const { return 0; }
      inline double operator( ) ( float f ) const { return f*f; }
   };

   template< >
   struct normdef< long double >
   {
      inline long double zero( ) const { return 0; }
      inline long double operator ( ) ( long double d ) const { return d*d; }
   };


   // Quaternion has no default constructor: 

   template< >
   struct normdef< quaternion >
   {
      inline quaternion zero( ) const { return {0,0,0,0}; }
      inline double operator( ) ( quaternion q ) const { return q. norm( ); } 
   };

} 

#endif

