
#include "tests.h"
#include "rigid.h"
#include "../units.h"

#include <iomanip>

/////////////////////////////////////////////////////////////////////
//                   Rigid Object Tests.                           //
/////////////////////////////////////////////////////////////////////


void physics::test_rigid( )
{

#if 1
   std::vector< pointmass > m =
      { { 1.0, { 0, 0, 0 } }, 
        { 2.0, { 0, 2, 0 } },
        { 3.0, { 1, 3, 0 } }, 
        { 4.0, { 2, 4, 7 } } };

   linalg::rototrans xxxx = { { 1, 0, 0, 0 }, { 0, 0, 0 } };
   linalg::spinspeed vvvv = { { 0, 0, 0 }, { 0, 0, 0 } };

   physics::rigid rig( xxxx, vvvv, m );

   std::cout << rig << "\n";
#endif

#if 0
   // Initialize with a spinning gyroscope:

   std::vector< pointmass > m =
   {
      { 1, { 0, 0, 1 } },
      { 1, { 0, 0, -1 } }, 
      { 1, { 1, 0, 0 } }, 
      { 1, { 0, 1, 0 } }, 
      { 1, { -1, 0, 0 } },
      { 1, { 0, -1, 0 } }
   };

   linalg::rototrans xxxx = { { 1, 0, 0, 0 }, { 0, 0, 0 } };
   linalg::spinspeed vvvv = { { 0, 0, 0 }, { 0, 0, 0 } };

   physics::rigid rig( xxxx, vvvv, m );
   std::cout << rig << "\n";
#endif 

#if 0
   // A flat example:

   std::vector< pointmass > m =
      { { 1, {0,0,0} },
        { 2, {0,3,0} },
        { 3, {1,3,0} },
        { 5, {2,4,0} } };

   linalg::rototrans xxxx = 
      { linalg::quaternion::rotation( { 0,0, 12.0_deg } ), { 1,-1,0 } };
   linalg::spinspeed vvvv = { { 0,0, -1.3 }, { 2,1,0 } };

   rigid rig( xxxx, vvvv, m ); 
   std::cout << rig << "\n";
#endif

   double h = 1.0E-9;
   double t0 = 0.0;
   double t1 = 6.0;
 
   double showtime = t0;
   while( t0 < t1 ) 
   {
      if( t0 >= showtime )
      {
         std::cout << "---------------------------------\n";
         std::cout << "t0 = " << t0 << "\n";
         std::cout << rig << "\n";

         showtime += 0.1;
      }

      force_torque ft = rig. effect_internal( 
                     rig.xxxx.q. inverse( ). rotate( { -5,2,0 } ), 
                     m[2]. position ); 

      if( t0 + h > t1 )
         h = t1 - t0;  

      rig. integrate( rig. accelerations( ft ), h );
 
      t0 += h; 
   }

   std::cout << "----------------------------\n";
   std::cout << "t = " << t0 << "\n";
   std::cout << rig << "\n";

   auto trans = rig. int2ext( );
   auto speed = rig. intspeed( );

   std::cout << "Positions and External Speeds of Rigid Object:\n";
   for( const auto& p : m )
   {
      std::cout << std::setprecision(15) << trans( p. position ) << "  ";
      std::cout << std::setprecision(15) << rig. speed2ext( speed( p. position )) << "\n";
   }

   std::cout << "\n";
   std::cout << trans << "\n";
   std::cout << speed << "\n\n";

#if 0

   linalg::vector v = linalg::vector(0,0,0);
   std::cout << "testpoint:   " << v << "\n";
   std::cout << "\n"; 
   std::cout << "position:    " << rig. position. apply(v) << "\n";
   std::cout << "speed:       " << rig. speed. apply(
                                    rig. position. apply(v)) << "\n";
   std::cout << "\n";

   // Purpose of this code is to test internal2speed and 
   // internal2wind: 
   
   {
      h = 1.0E-6; 

      // We calculate how p moves from x0 to x1 in short time h.

      linalg::vector x0 = rig. position. apply(v); 

      physics::force_torque nullforce( rig. position. v ); 
      rig. integrate( h, nullforce );

      linalg::vector x1 = rig. position. apply(v);

      std::cout << "speed by differentation:   ";
      std::cout << ( x1 - x0 ) / h << "\n";

      std::cout << "rig. speed tells       :   " << rig. speed. apply(x0);
      std::cout << "\n\n";  

      // Same procedure for internal2wind:

      x0 = rig. position. inverse( ). apply(v);
      rig. integrate( h, nullforce );
      x1 = rig. position. inverse( ). apply(v);
      std::cout << "wind speed by differentiating:            ";
      std::cout << ( x1 - x0 ) / h << "\n";

      std::cout << "windspeed by rig. internalspeedfunc( ):   ";
      std::cout << - rig. internalspeedfunc( ). apply( x0 );
      std::cout << "\n\n"; 
#endif 
}


