
// Written by Hans de Nivelle, November 2014.

#include "plotwindow.h" 
#include <iostream>
#include <sstream>

#include "graphics.h" 
#include "linalg/fourmatrix.h"

bool 
plotwindow::handle_events( std::function< bool( const sf::Event& ) > handler )
{
   sf::Event event;
   while( window. pollEvent( event ))
   {
      // State indepent events:

      if( event. type == sf::Event::Closed )
      {
         window. close( );
         std::cout << "Window was closed\n";
         return false;
      }

      if( event.type == sf::Event::Resized )
      {
         glViewport( 0, 0, event.size.width, event.size.height );
         goto handled; 
      }

      // State independent Key presses:

      if( event.type == sf::Event::KeyPressed )
      {
         if( event.key.code == sf::Keyboard::Escape )
         {
            window. close( );
            std::cout << "Escape pressed, Window closed\n";
            return false;
         }

         if( event.key.code == sf::Keyboard::Home )
         {
            resizing = !resizing; 
            goto handled; 
         }
      }

      // If we are not resizing and we have a handler, we call it: 

      if( !resizing && handler ) 
      {
         auto b = handler( event );
         if( !b ) return false;
         goto handled;  
      }

      if( event.type == sf::Event::KeyPressed )
      {
         switch( event.key.code )
         {
         case sf::Keyboard::Subtract: 
            xhalfsize *= factor; yhalfsize *= factor; break;

         case sf::Keyboard::Add: 
            xhalfsize /= factor; yhalfsize /= factor; break;

         case sf::Keyboard::Left: 
            xcenter -= xhalfsize / 5; break;

         case sf::Keyboard::Right:
            xcenter += xhalfsize / 5; break;

         case sf::Keyboard::Up:
            ycenter += yhalfsize / 5; break;

         case sf::Keyboard::Down:
            ycenter -= yhalfsize / 5; break;

         case sf::Keyboard::Numpad8:
            yhalfsize /= factor; break;

         case sf::Keyboard::Numpad2:
            yhalfsize *= factor; break;

         case sf::Keyboard::Numpad5:
            xcenter = xcenterstart;      ycenter = ycenterstart;
            xhalfsize = xhalfsizestart;  yhalfsize = yhalfsizestart;
            break;
         }
         goto handled; 
      }
      handled:
         ;
   }

   return true; 
}


void plotwindow::prepare( bool grid )
{
   graphics::preparefor2d( );

   graphics::setview(
       linalg::fourmatrix::orthographic( 
                   xcenter - xhalfsize, xcenter + xhalfsize,
                   ycenter - yhalfsize, ycenter + yhalfsize,
                   1, -1 ));

   if( grid ) 
   {
      graphics::drawgrid( xcenter - xhalfsize, xcenter + xhalfsize,
                          ycenter - yhalfsize, ycenter + yhalfsize );
   }

   std::ostringstream ss;
   ss << header << ": ";
   ss << "x in [ " << xcenter - xhalfsize << ", ";
   ss << xcenter + xhalfsize << " ] ";

   ss << ",    ";
   ss << "y in [ " << ycenter - yhalfsize << ", ";
   ss << ycenter + yhalfsize << " ]";

   if( resizing )
      ss << "        ( RESIZING RESIZING RESIZING RESIZING )"; 

   window. setTitle( ss. str( ));
}


void plotwindow::plot( const std::function< double( double ) > & f,
                       scenery::color col ) 
{
   graphics::plot( f, xcenter - xhalfsize, xcenter + xhalfsize, col );
}


