
// Plotwindow is intended for plotting graphs of functions.
// November 2014.
// Plotwindow should be used only for plotting flat things, not
// for plotting 3D views. I will later write a separate class for that.
// Maybe I will also extend this class, it seems to be possible.

#ifndef PLOTWINDOW_INCLUDED
#define PLOTWINDOW_INCLUDED    1

#include <SFML/System.hpp>
#include <SFML/Window.hpp>
#include <SFML/OpenGL.hpp>

#include <stdexcept> 
#include <functional> 
#include <cmath>

#include "color.h"

struct plotwindow
{
   std::string header;
   sf::Window window; 
   bool resizing;     // if true, events are used for resizing. 
                      // User can toggle by pressing Home.

   double xcenter, ycenter;
   double xhalfsize, yhalfsize; 

   double xcenterstart, ycenterstart;
   double xhalfsizestart, yhalfsizestart;
      // The user can press NUM 5 to restore the original settings.

   static constexpr double factor = pow( 10, 1.0 / 4.0 );
   
   plotwindow( const std::string& header = "Graph", 
               double x0 = -1, double x1 = 1, 
               double y0 = -1, double y1 = 1,
               unsigned int xpixels = 1024, unsigned int ypixels = 600 )
      : header{ header },
        window{ sf::VideoMode( xpixels, ypixels, 32 ), header },
        resizing{ false }, 
        xcenter{ ( x0 + x1 ) / 2 },      ycenter{ ( y0 + y1 ) / 2 },
        xhalfsize{ ( x1 - x0 ) / 2 },    yhalfsize{ ( y1 - y0 ) / 2 },
        xcenterstart{ xcenter }, ycenterstart{ ycenter },
        xhalfsizestart{ xhalfsize }, yhalfsizestart{ yhalfsize }
   {
      if( xhalfsize <= 0.0 )
         throw std::runtime_error( "x0 must be less than x1" ); 
      if( yhalfsize <= 0.0 )
         throw std::runtime_error( "y0 must be less than y1" ); 

      window. setFramerateLimit(25);
      window. display( );
   }

   void setcenter( double x, double y )
   {
      xcenter = x;
      ycenter = y;
   }
      
   bool handle_events( 
            std::function< bool( const sf::Event& ) > handler = nullptr ); 
      // Check for events, and carry them out.
      // Returning false means that the user expressed desire to quit.
      // Plotwindow can be in two modes:
      // In resizing mode, events are used for resizing or moving the
      // perspective, and nothing is passed to handler. 
      // In application mode, most events are passed to handler, which 
      // can decide what to do with it. handler can return false if the
      // user wants to quit. Otherwise, it must return true.  

   void prepare( bool grid = true ); 
      // Set OpenGL, clear the screen, and set a suitable
      // orthographic projection. If grid == true,
      // then we also draw a grid.

   void plot( const std::function< double( double ) > & f,
              scenery::color col ); 

   void plot( const std::function< std::pair<double,double> ( double ) > & f,
              double t0, double t1, scenery::color col );
      // f is a function t -> (x,y), which will be plotted in the
      // the range t0 .. t1.

   void display( )
   {
      window. display( );
   }

};

#endif

