
#include "twodimensional.h"
#include "linalg/matrix.h"

force_effect twodimensional::fuselage( ) const
{
   // Of course we do have no clue what we are doing, and we don't 
   // know if it makes any sense. The main thing is that we manage
   // to return some value. 

   double alpha = theta + atan2( -v0.y, v0.x );
   double q = 0.5 * rho * v0. norm( );
      // (1/2) rho V^2.

   // std::cout << "alpha-fuselage " << alpha / 1.0_deg << "\n";
   // std::cout << "q-fuselage = " << q << "\n";

   coefficients c =
      ( alpha >= 0 ) ?
            fuselage_coefficients( alpha ) :
            fuselage_coefficients( -alpha ). upsidedown( );

   // std::cout << "coefficients " << c << "\n";

   linalg::vector f =
      q * ( fuselage_surfaces.y * c. lift * linalg::vector( -v0.y, v0.x, 0 ) -
            fuselage_surfaces.x * c. drag * v0 ) / v0. length( );

   double t = q * fuselage_surfaces.y * fuselage_length * c. moment;
      // Since we have no idea what the moment from drag might be,
      // we assume that it is 0.

   auto fuselageposworld =
      linalg::vector(
         fuselage_position.x * cos(theta) - fuselage_position.y * sin(theta),
         fuselage_position.x * sin(theta) + fuselage_position.y * cos(theta),
         0 );

   t = t + fuselageposworld.x * f.y - fuselageposworld.y * f.x;

   return { f, t };
}


force_effect twodimensional::main_wings( ) const
{
   // We determine angle of attack: 

   double alpha = theta + wing_theta + atan2( -v0.y, v0.x );
      // We are assuming that the main wings are so close to center of mass,
      // that omega does not matter.
   
   double q = 0.5 * rho * v0. norm( );
      // (1/2) rho V^2.

   // std::cout << "alpha-main wing " << alpha / 1.0_deg << "\n";
   // std::cout << "q-main wing = " << q << "\n";

   coefficients c = wing_coefficients( alpha );
   // std::cout << "coefficients-main wing " << c << "\n";

   linalg::vector f = 
      q * wing_surface *
         ( c. lift * linalg::vector( -v0.y, v0.x, 0 ) - 
           c. drag * v0 ) / v0. length( );

   double t = q * wing_surface * wing_chord * c. moment; 

#if 0
   auto pres = c. pressure_center( );
   std::cout << "pressure center " << pres << "\n";
   std::cout << "alternative computation, should be the same: ";
   std::cout << force_effect( f, 
      wing_position + 
         linalg::vector( -cos( theta + wing_theta ),
                         -sin( theta + wing_theta ), 0 ) * pres * wing_chord ) 
         << "\n";
#endif 
   // std::cout << "force = " << f << "\n";
   // std::cout << "torque = " << t << "\n";

   // Wing Position in world coordinates, see slides. 
   // This was corrected on 10 december 2016.

   auto wingposworld =
      linalg::vector( 
         wing_position.x * cos( theta ) - wing_position.y * sin( theta ), 
         wing_position.x * sin( theta ) + wing_position.y * cos( theta ),
         0 );

   t = t + wingposworld.x * f.y - wingposworld.y * f.x;

   return { f, t };
}


force_effect twodimensional::elevators( ) const 
{
   // First we determine angle of attack: 

   // Since elevators are far away from the mass center,
   // their speed is affected by the angular velocity of the
   // plane.

   // This is just v0 + omega X e, where e is the current
   // elevator position, which depends on theta:

   linalg::vector w = 
      { v0.x - omega * ( elevator_position.x * sin(theta) +
                         elevator_position.y * cos(theta) ),
        v0.y + omega * ( elevator_position.x * cos(theta) -
                         elevator_position.y * sin(theta) ),
        0.0 };

   // std::cout << "linear speed = " << v0 << "\n";
   // std::cout << "omega = " << omega << "\n";
   // std::cout << "local speed of elevator " << w << "\n";
   // std::cout << "         " << w. length( ) << "\n"; 

   double alpha = theta + elevator_theta + atan2( -w.y, w.x );
   // std::cout << "theta-elevator " << elevator_theta / 1.0_deg << "\n";
   // std::cout << "alpha-elevator " << alpha / 1.0_deg << "\n";

   double q = 0.5 * rho * w. norm( );
      // (1/2) rho V^2.

   // std::cout << "q-elevator = " << q << "\n";

   coefficients c = 
      ( elevator_deflection >= 0 ) ? 
         coefficients::interpolate( elevator_deflection,
                            elevator_coefficients0(alpha), 
                            elevator_coefficients1(alpha) ) :
         coefficients::interpolate( -elevator_deflection,
                            elevator_coefficients0(alpha),
                            elevator_coefficients1(-alpha). upsidedown( )); 
  
   // std::cout << "elevator deflection " << elevator_deflection << "\n";
   // std::cout << "elevator coefficients " << c << "\n\n";      
   
   linalg::vector f =
      q * elevator_surface *
         ( c. lift * linalg::vector( -w.y, w.x, 0 ) -
           c. drag * w ) / w. length( );

   double t = q * elevator_surface * elevator_chord * c. moment;

   auto elevatorposworld =
      linalg::vector(
         elevator_position.x * cos(theta) - elevator_position.y * sin(theta),
         elevator_position.x * sin(theta) + elevator_position.y * cos(theta),
         0.0 );

   t = t + elevatorposworld.x * f.y - elevatorposworld.y * f.x;

   return { f, t };

}


std::ostream& operator << ( std::ostream& out, const force_effect& f )
{
   out << "force_effect( " << f. linear << ", " << f. torque << " )";
   return out;
}


std::ostream& operator << ( std::ostream& out, const twodimensional& t )
{
   out << "position: " << t. position << "\n";
   out << "theta: " << t. theta / 1.0_deg << " deg\n";
   out << "v0 " << t. v0 << "\n";
   out << "omega " << t. omega << "\n";
   out << "throttle " << t. throttle << "\n";
   out << "engine power " << t. engine_power * 100 << " pct\n";
   out << "elevator deflection " << t. elevator_deflection << "\n";
   out << "elevator theta " << t. elevator_theta / 1.0_deg << " deg\n";
   return out;
}

 
